<?php  /* Hey, emacs!  Edit this file as -*- mode: php; coding: utf-8 -*- */
  /* $Id: onephoto.php,v 1.220 2012/07/09 04:32:20 grog Exp $ */

$debug = 0;                                     /* set to enable debugging code */
$showinfo = 0;                                  /* override "1" and show exposure info */
$photoseq = -1;                                 /* sequence number of photo on this page */
$edit = 0;                                      /* set to 1 if we want to edit things */
$imagelevel = 1;                                /* show all images until proof of the contrary */
$needimagepath = 0;                             /* normally we don't need to specify an image path in URLs */
$crawled = iscrawler ();                        /* set if we're being crawled */
$loadimage = "/grog/Photos/20120628/tiny/loading-image.gif"; /* image to display while loading the real one */

/*
 * This file uses a number of variables that should be propagated via
 * self-referent links if, and only if, they're not the default.  They're
 * described in more detail further down.
 *
 * XXX These parameters are currently subject to review.  In particular, dirdate
 * is only needed for photos.php, and it's actively wrong for diaries.  image
 * only seems to be needed for onephoto.php.
 *
 * size        Image size, from 0 (hidden) to 3 (full original size).
 *             Default 1
 * imagesizes  String giving sizes of individual images on page.
 *             Default empty
 * image       Relative pathname of image
 *             Default empty
 *             XXX I'm not sure we need this; check some time
 * edit        (local only)
 *             Default empty
 * showdetails Details to show, under development
 *             Default empty
 * topics      Diary topics (see header.php)
 *             Default empty
 * dirdate     Date string in form YYYYMMDD
 *             No default.
 * view        Forgotten what this is, probably for house photos
 *             Default empty
 */

/* Ensure that we have a valid size specification */
if (array_key_exists ("size", $_GET))
{
  $defaultsize = $_GET ["size"];
  /* Convert old sizes */
  if ($defaultsize == "tiny")
    $defaultsize = 1;
  if ($defaultsize == "small")
    $defaultsize = 2;
  else if ($defaultsize == "big")
    $defaultsize = 3;
  else if ((strlen ($defaultsize) > 1)
           || ($defaultsize < "0")
           || ($defaultsize > "4") )
    $defaultsize = 1;
}
else if ($crawled)
  $defaultsize = "4";
else
  $defaultsize = "1";
/* Some clients to showphoto () have a ?view= parameter */
if (! isset ($view))
  $view = "";

/*
 * Decide on the size of the images on the page.
 *
 * $imagesizes is a "bit map" (really a string) of sizes indexed
 * by $photoseq.
 *
 * There are five sizes:
 *
 * 0: Don't display at all
 * 1: "tiny", from the tiny/ directory.
 * 2: "small", from the small/ directory.
 * 3: "big", from the big/ directory, to fit width of window.
 * 4: "big", from the big/ directory, full width.
 *
 * If we run out, go by $defaultsize.
 *
 * If this is a request from a web crawler, we handle it differently:
 *
 * 1: If the crawler specifies an imagesizes parameter, redirect to one without
 *    the parameter.
 * 2: Otherwise give it the largest possible images (big).  This is done in
 *    displayargs ().
 */

if (array_key_exists ("imagesizes", $_GET))
  {
    if ($crawled)                               /* redirect to stop multiple crawls  */
    {
      print "<pre>";
      print_r ($_SERVER);
      print "</pre>";

      $REDIRECT = $_SERVER ["SCRIPT_NAME"];
      /* XXX kludge.  We should extract these variables globally. */
      if (array_key_exists ("dirdate", $_GET))
        $REDIRECT .= "?dirdate=" . $_GET ["dirdate"];
      $URL = $_SERVER ["REQUEST_URI"];
      $SERVER = $_SERVER["HTTP_HOST"];
      $FULLURL = "http://" . $_SERVER ["HTTP_HOST"] . $URL;
      if (0)
        mail ($_SERVER ["SERVER_ADMIN"],
              "Crawler request from $SERVER",
              <<< EOS
Referenced URL: $FULLURL
Redirected to:  $REDIRECT
EOS
              );
      redirect ($REDIRECT);
    }
    else
    {
      /*
       * XXX Make sure we translate the old chars to the new.  If I had known how
       * many of these things get bookmarked, I'd never have changed them.
       */
      $oldimagesizes = $_GET ["imagesizes"];
      $imagesizes = strtr ($oldimagesizes, "tsbB", "1234");
    }
  }
else if ($crawled)                              /* not set, being crawled */
  $imagesizes = $oldimagesizes = "4";           /* we run out right at the start */
else                                            /* not set, not being crawled */
  $imagesizes = $oldimagesizes = "";            /* we run out right at the start */

/*
 * Size specs for each photo.
 */
if (! isset ($photodetails))
{
  if (array_key_exists ("photodetails", $_POST))
    $photodetails = $_POST ["photodetails"];
  else
    $photodetails = array ();
}

/*
 * Decide whether to redirect crawler access to individual JPEGs.
 * XXX This is a good idea, but it doesn't work: we don't get invoked for JPEGs.
 * Left here as a reminder.
 */

if ($crawled)
{
  $URI = $_SERVER ["REQUEST_URI"];

  if (preg_match (":/tiny/|/small/:", $URI))
  {
    $URI = preg_replace (":/tiny/|/small/:", "/big/", $URI);
    redirect ($URI);
  }
}

/*
 * Build up a parameter list to pass to next URL.
 *
 * Parameters:
 *
 * $size is the size we want for this image.
 * $imagesizes is a mask of sizes for up to all images on the page.
 * $dirdate is the date of this image.
 * $dammit is set if we want to return the $dirdate even for a diary.  By
 *          default, we don't return the image path for a diary, but in some
 *          cases it's necessary.
 */
function displayargs (/* $size, $imagesizes, $dirdate, $dammit */)
{
  global $size;
  global $imagesizes;
  global $thisimage;
  global $imagepath;
  global $showdetails;
  global $topics;
  global $dirdate;
  global $view;
  global $edit;
  global $isdiary;
  global $needimagepath;
  global $keywords;
  global $crawled;                      /* set if we're being crawled */
  global $debug;

  $delim = "?";                                 /* delimiter before arg */
  $args = "";                                   /* to return */
  $dammit = ! $isdiary;                         /* show dirdate if we're not a diary */
  $photodir = $dirdate;                         /* default to current directory date */

  /* get optional args */
  $myimagesizes = $imagesizes;
  switch (func_num_args ())
  {
  case 0:
    $mysize = $size;
    break;

  default:                                      /* 4 or more */
    $dammit = 1;                                /* we want the dirdate even if we're a diary */

    /* FALLTHROUGH */
  case 3:                                       /* specific directory date */
    $photodir = func_get_arg (2);               /* get it */

    /* FALLTHROUGH */
  case 2:
    $myimagesizes = func_get_arg (1);

    /* FALLTHROUGH */
  case 1:
    $mysize = func_get_arg (0);
  }

  if ($debug)
    print "<pre>mysize $mysize imagesizes $myimagesizes photodir $photodir showdetails $showdetails thisimage $thisimage dammit $dammit</pre>";
  if ($mysize == "")
    $mysize = "1";                              /* to simplify some tests */

  /*
   * Don't set dirdate for diaries.  This is arguably wrong, since we really
   * only need it for the photo pages, but it doesn't do any harm elsewhere.
   */
  if (($photodir != "") && $dammit)
  {
    $args .= "$delim" . "dirdate=$photodir";
    $delim = "&amp;";
  }

  /*
   * Do we need to specify the path of the image?
   */
  if ($needimagepath)
  {
    if ($imagepath != "")
    {
      $args .= "$delim" . "image=$imagepath";
      $delim = "&amp;";
    }
    else if ($thisimage != "")
    {
      $args .= "$delim" . "image=$thisimage";
      $delim = "&amp;";
    }
  }
  if ($mysize != "1")                           /* not default */
  {
    $args .= "$delim" . "size=$mysize";
    $delim = "&amp;";
  }
  if ($myimagesizes != "" && ! $crawled)
  {
    /*
     * Check whether $myimagesizes differs from $mysize; if not, we don't need it.
     */
    $differs = 0;
    for ($i = 0; $i < strlen ($myimagesizes); $i++)
      if (substr ($myimagesizes, $i, 1) != $mysize)
      {
        $differs = 1;
        break;
      }
    if ($differs)
    {
      $args .= "$delim" . "imagesizes=$myimagesizes";
      $delim = "&amp;";
    }
  }
  if ($showdetails != "")
  {
    $args .= "$delim" . "showdetails=$showdetails";
    $delim = "&amp;";
  }
  if ($topics != "")
  {
    $args .= "$delim" . "topics=$topics";
    $delim = "&amp;";
  }
  if ($view != "")
  {
    $args .= "$delim" . "view=$view";
    $delim = "&amp;";
  }
  if ($edit != "")
  {
    $args .= "$delim" . "edit=$edit";
    $delim = "&amp;";
  }
  if (isset ($keywords) && $keywords != "")
  {
    $args .= "$delim" . "keywords=$keywords";
    $delim = "&amp;";
  }
  return $args;
}

/*
 * Refer to a set of photos.  Parameter is an 8 digit date.
 * $photo_id, if present, is a tag to append to the URL.
 * $who, if present, is the TLD for the photos.
 */
function photoref ($date, $text /* , $photo_id, $who */)
{
  global $who;
  global $servername;                           /* name of server, with http:// */

  $tag = "";
  /* this mess is to allow for empty tag paramters */
  if (func_num_args () > 2)
    $tag = func_get_arg (2);
  if ($tag != "")
    $tag = "#photo-$tag-$date";

  /* whose photos? */
  if (func_num_args () > 3)
    $whose = func_get_arg (3);
  else
    $whose = $who;

  print "<a href=\"$servername/$whose/photos/Photos.php?dirdate=$date$tag\">$text</a>";
}

/*
 * Refer to a single photo $image, with $text as link text.
 * Note that this doesn't imply directory Photos.
 * $who, if present, is the TLD for the photos.
 */
function onephotoref ($image, $text /* , $who */)
{
  global $defaultsize;
  global $who;
  global $servername;                           /* name of server, with http:// */

  /* whose photos? */
  if (func_num_args () > 2)
    $whose = func_get_arg (2);
  else
    $whose = $who;

  print "<a href=\"$servername/$whose/photos/Onephoto.php?image=$image&amp;size=$defaultsize\">$text</a>";
}

/*
 * Get the most interesting exposure information from an EXIF file and return it
 * as an HTML string.
 *
 * This is a heuristic attempting to interpret the output of exif (which uses |
 * as a delimiter) and exiftool (which uses : as a delimiter).  Both are
 * strongly influenced by individual camera models, so some of the tests look
 * strange and may break for cameras I haven't seen.
 *
 * $format is:
 *    "c" for compact layout (single column, <td> delimiters)
 *    "t" for tabular layout (two columns, <tr><td> delimiters)
 *    "j" for Javascript format, effectively the same as the t layout, but
 *         without line breaks to make it more illegible
 *     "p" or anything else for plain text.
 */
function exposuredetails ($exif_file, $format)
{
  global $bigwidth;
  global $bigheight;
  global $bigkb;

  $manufacturer = "";
  $model = "";
  $date = "";
  $author = "";
  $shutter = "";
  $aperture = "";
  $ISO = "";
  $light_value = 0;
  $metermode = "";
  $EV = "";
  $exposure_program = "";
  $flash = "";
  $flash_mode = "";
  $flash_comp = "";
  $external_flash = "";
  $external_flash_bounce = "";
  $external_flash_zoom = "";
  $manual_flash = "";
  /*
   * These two together determine the Mecablitz 58 AF 1 unit: it reports itself
   * as an Olympus FL-50R with a serial number 000058:1.
   */
  $flash_model = "";
  $flash_serial = "";
  $focal_length = "";
  $scale_factor = "";
  $equivalent_focal_length = "";
  $extender = "None";                           /* default return value from exiftool */
  $saturation = "";
  $lens = "";
  $focus = "";
  $focus_mode = "";
  $depth_of_field = "";
  $details = "";                                /* fill details in here */
  $scale_factor = 0;                            /* scale factor to 35 mm */
  $projection = "";                             /* for panoramas */

  /* Variables for calculating FoV */
  $sensorheight = 0;
  $sensorwidth = 0;
  $cropheight = 0;
  $cropwidth = 0;
  $horzcrop = 1;                                /* horizontal and */
  $vertcrop = 1;                                /* vertical crop factors */
  $horizontal_fov = 0;
  $vertical_fov = 0;

  /* Decide how we're going to lay this stuff out */
  switch ($format)
  {
  case "c":                                     /* compact layout */
    $opent = "              <tr><td>";
    $sep = " ";
    $closet = "</td></tr>\n";
    $deg = "°";
    $inf = "&infin;";
    break;

  case "j":                                     /* Javascript layout */
    $opent = "<tr><td>";
    $sep = "<\\/td><td>";
    $closet = "<\\/td></tr>";
    $deg = "°";
    $inf = "&infin;";
    break;

  case "t":
    $opent = "                <tr><td>";
    $sep = "</td><td>";
    $closet = "</td></tr>\n";
    $deg = "°";
    $inf = "&infin;";
    break;

  default:
    $format = "p";
    $opent = "";
    $sep = "\t";
    $closet = "\n";
    /* This usually gets output in ISO-8859.  FIXME.*/
    /* $deg = "°"; */
    $deg = "";
    $inf = "infinity";
    break;
  }

  $fd = fopen ("$exif_file", "r");
  while ($info = fgets ($fd))
  {
  if ($author == "")
    {
      if (preg_match ("~^Author *:(.*)~", $info, $parts) || preg_match ("~^Artist *:(.*)~", $info, $parts))
        $author = trim ($parts [1]);
    }
    if ($manufacturer == "")
    {
      if ((preg_match ("~^Make *: (.*)~", $info, $parts))
          || (preg_match ("~^Manufacturer *\|(.*)~", $info, $parts)) )
        $manufacturer = trim ($parts [1]);
    }
    if ($model == "")
    {
      if ((preg_match ("~^Camera Model Name *:(.*)~", $info, $parts))
          || (preg_match ("~^Model *\|(.*)~", $info, $parts)) )
        $model = trim ($parts [1]);
    }
    if ($date == "")
    {
      if (preg_match ("~^Date/Time Original *:(.*)~", $info, $parts)
          || preg_match ("~^Date and Time.*ori.*\|(.*)~", $info, $parts) )
      {
        /*
         * Some cameras, notably my Nikon "CoolPix" 880, return invalid dates
         * after removing the batteries.  Catch at least this case.
         */
        $rawdate = trim ($parts [1]);
        if (! preg_match ("~0000:00~", $rawdate))
          $date = formatdate ("l, j F Y, G:i:s", validdate ($rawdate, ""));
      }
    }
    if ($shutter == "")
    {
      if (preg_match ("~^Exposure Time *:(.*)~", $info, $parts))
        $shutter = trim ($parts [1]) . " sec";
      else if (preg_match ("~^Exposure Time *\|(.*)~", $info, $parts))
        $shutter = trim ($parts [1]);
    }
    if ($aperture == "")
    {
      if (preg_match ("~^Aperture *: (.*)~", $info, $parts))
        $aperture = "f/" . trim ($parts [1]);
      else if (preg_match ("~^FNumber *\|(.*)~", $info, $parts))
        $aperture = trim ($parts [1]);
      if ($aperture == "f/0.0")                 /* unknown */
        $aperture = "<i>aperture unknown</i>";
    }
    if  ($ISO == "")
    {
      if (preg_match ("~^ISO  *: (.*)~", $info, $parts)
          || (preg_match ("~^ISO Speed Ratings *\|(.*)~", $info, $parts)) )
      {
        $ISO = trim ($parts [1]);
        $ISOgrad = log10 ($ISO) * 10 + 1;
        /*
         * This fudge factor is needed because the ISO arithmetic values differ
         * considerably from the pure logarithmic progression.
         */
        if (($ISOgrad - floor ($ISOgrad)) > 0.07) /* not a real value */
          $ISO = sprintf ("%s/%3.1f$deg", $ISO, $ISOgrad);
        else
          $ISO = sprintf ("%s/%d$deg", $ISO, $ISOgrad);
      }
    }
    if  ($light_value == 0)
    {
      if (preg_match ("~^Light Value  *: (.*)~", $info, $parts))
        $light_value = trim ($parts [1]);
    }
    if ($metermode == "")
    {
      if (preg_match ("~^Metering Mode *[|:] *(.*)~", $info, $parts))
        $metermode = trim ($parts [1]);
    }
    if  ($exposure_program == "")
    {
      if (preg_match ("~^Exposure Program *: (.*)~", $info, $parts))
        $exposure_program = trim ($parts [1]);
    }
    if  ($EV == "")
    {
      if (preg_match ("~^Exposure Compensation *: (.*)~", $info, $parts))
        $EV = trim ($parts [1]) . " EV";
      else if (preg_match ("~^Exposure Bias *\|(.*)~", $info, $parts))
        $EV = trim ($parts [1]);
    }
    if ($flash == "")
    {
      if (preg_match ("~^Flash *[|:] *(.*)~", $info, $parts))
      {
        $flash = trim ($parts [1]);
        /* Sometimes only we get "On,  " at the beginning here */
        $flash = preg_replace ("|On, (.*)|", "\\1", $flash);
        /* We don't want to know about non-events */
        if ($flash == "Did not fire")
          $flash = "";
      }
    }
    if ($flash_mode == "")
    {
      if (preg_match ("~^Flash Mode *: (.*)~", $info, $parts))
      {
        $flash_mode = trim ($parts [1]);
        /* Sometimes only we get "On,  " at the beginning here */
        $flash_mode = preg_replace ("|On, (.*)|", "\\1", $flash_mode);
      }
    }
    if ($flash_model == "")
    {
      if (preg_match ("~^Flash Model *: (.*)~", $info, $parts))
        $flash_model = trim ($parts [1]);
    }
    if ($flash_serial == "")
    {
      if (preg_match ("~^Flash Serial Number *: (.*)~", $info, $parts))
        $flash_serial = trim ($parts [1]);
    }
    if ($flash_comp == "")
    {
      if (preg_match ("~^Flash Exposure Comp *: (.*)~", $info, $parts))
      {
        $flash_comp = trim ($parts [1]);
        if (substr ($flash_comp, 0, 1) != "-")
          $flash_comp = "+$flash_comp";
      }
    }
    if ($manual_flash == "")
    {
      if (preg_match ("~^Manual Flash *: On (.*)~", $info, $parts))
        $manual_flash = trim ($parts [1]);
    }
    if ($external_flash == "")
    {
      if (preg_match ("~^External Flash *: (.*)~", $info, $parts))
        $external_flash = trim ($parts [1]);
      if ($external_flash == "Off")             /* reported, but not there */
        $external_flash = "";
    }
    if ($external_flash_bounce == "")
    {
      if (preg_match ("~^External Flash Bounce *: (.*)~", $info, $parts))
        $external_flash_bounce = trim ($parts [1]);
      if ($external_flash_bounce == "Off")
        $external_flash_bounce = "";
    }
    if ($external_flash_zoom == "")
    {
      if (preg_match ("~^External Flash Zoom *: (.*)~", $info, $parts))
      {
        if (($external_flash_zoom = trim ($parts [1])) == "0")
          $external_flash_zoom = "";
      }
    }
    if ($saturation == "")
    {
      if (preg_match ("~^Saturation *:(.*)~", $info, $parts))
        $saturation = trim ($parts [1]) . " saturation";
      else if (preg_match ("~^Saturation *\|(.*)~", $info, $parts))
        $saturation = trim ($parts [1]);
    }
    if ($focus == "")
    {
      if (preg_match ("~^Focus Distance *:(.*)~", $info, $parts))
        $focus = trim ($parts [1]);
    }
    /* XXX focus mode appears to be reported incorrectly */
    if ($focus_mode == "")
    {
      if (preg_match ("~^Focus Mode *:([ A-Z]*)~", $info, $parts))
        $focus_mode = trim ($parts [1]);
    }
    if ($depth_of_field == "")
    {
      if (preg_match ("~^Depth Of Field *:.*(\(.*\))~", $info, $parts))
        $depth_of_field = trim ($parts [1]);
    }
    /*
     * At least for the Olympus E-510, exiftool returns two different "Focal
     * Length" entries.  They have the same value (fortunately!), but the second
     * one includes a rough full frame (35 mm) equivalent.  This code attempts
     * to extract that info, so we don't check if $focal_length is already set.
     * On the other hand, we can calculate the equivalent focal length better
     * than exiftool, so if we find a "Scale Factor To 35 mm Equivalent", we'll
     * take that in preference.
     */
    if (preg_match ("~^Focal Length *[|:](.*)~", $info, $parts))
    {
      if ($focal_length == "")
        $focal_length = trim ($parts [1]);
      if (($equivalent_focal_length == "")
          && (preg_match ("~\(35 mm equivalent: ([^0].*)\)~", $info, $parts)) )
        $equivalent_focal_length = trim ($parts [1]);
      if ($equivalent_focal_length == "0 mm")
        $equivalent_focal_length = "";
    }
    if (preg_match ("~^Scale Factor To 35 mm Equivalent *[:](.*)~", $info, $parts))
      $scale_factor = trim ($parts [1]);

    if (($extender == "None")
        && (preg_match ("~^Extender *: (.*)~", $info, $parts)) )
      $extender = trim ($parts [1]);
    /*
     * Get lens info.  Lens ID seems the most promising, so let that override.
     */
    if (preg_match ("~^Lens ID *: (.*)~", $info, $parts))
    {
      $lens = trim ($parts [1]);
      if (($lens == "None")                     /* no lens info, from Olympus */
          || ($lens == "Unknown (-1)") )        /* no lens info, from Canon */
        $lens = "";
    }
    if (($lens == "")
        && (preg_match ("~^Lens Type *: (.*)~", $info, $parts)) )
    {
      $lens = trim ($parts [1]);
      if (($lens == "None")                     /* no lens info, from Olympus */
          || ($lens == "Unknown (-1)") )        /* no lens info, from Canon */
        $lens = "";
    }
    /* No luck.  Try again with lens model. */
    if (($lens == "")
        && (preg_match ("~^Lens Model *: (.*)~", $info, $parts)) )
    {
      $lens = trim ($parts [1]);
      if (($lens == "None")                     /* no lens info, from Olympus */
          || ($lens == "Unknown (-1)") )        /* no lens info, from Canon */
        $lens = "";
    }
    /* This seems to happen on Pentaces */
    if ($equivalent_focal_length == "")
    {
      if (preg_match ("~^Focal Length In 35mm *\|(.*)~", $info, $parts))
      {
        $equivalent_focal_length = trim ($parts [1]);
        if ($equivalent_focal_length == "0.0 mm") /* none */
          $equivalent_focal_length = "";        /* reset */
      }
    }
    /*
     * Despite the naming here, "Crop Height" or "Exif Image Height" appear to
     * mean the dimensions of the sensor.
     */
    if (($sensorheight == 0)
        && (preg_match ("~^Crop Height *: (.*)~", $info, $parts)
            || preg_match ("~^Exif Image Height *: (.*)~", $info, $parts) ) )
      $sensorheight = trim ($parts [1]);
    if (($sensorwidth == 0)
        && (preg_match ("~^Crop Width *: (.*)~", $info, $parts)
            || preg_match ("~^Exif Image Width *: (.*)~", $info, $parts) ) )
      $sensorwidth = trim ($parts [1]);
    if (($cropheight == 0)
        && preg_match ("~^Image Height *: (.*)~", $info, $parts) )
      $cropheight = trim ($parts [1]);
    if (($cropwidth == 0)
        && preg_match ("~^Image Width *: (.*)~", $info, $parts) )
      $cropwidth = trim ($parts [1]);
    if (preg_match ("~^Scale Factor To 35 mm Equivalent *\|(.*)~", $info, $parts))
      $scale_factor = trim ($parts [1]);

    if (preg_match ("~^User Comment *: (.*)~", $info, $parts))
    {
      /* User Comment                    : .Projection: Cylindrical (1).FOV: 168 x 104.Ev: 10.72   */
      $comments = explode (".", trim ($parts [1]));
      foreach ($comments as $comment)
      {
        if (preg_match ("~^Projection: (.*) \(.*\)~", $comment, $parts))
          $projection = $parts [1];
        else if (preg_match ("~^FOV: (.*) x (.*)~", $comment, $parts))
        {
          $horizontal_fov = $parts [1];
          $vertical_fov = $parts [2];
        }
      }
    }
  }

  fclose ($fd);

  /* Now put the stuff together */
  /*
   * Can we calculate the 35 mm equivalent better?  If so, do so, even if we
   * already have a value.
   */
  if (($scale_factor != "")
      && ($focal_length != "") )
    $equivalent_focal_length = $scale_factor * $focal_length;
  if ($author != "")
    $details .= "$opent" . "Author: $sep$author$closet";
  $model = "$manufacturer $model";
  if ($date != "")
    $details .= "$opent" . "Date taken:$sep$date$closet";
  if ($shutter != "" || $aperture != "")
  {
    $details .= "$opent" . "Exposure:$sep$shutter";
    if ($aperture != "")
    {
      if ($shutter != "")                       /* Apple ipHones don't report shutter speeed */
        $details .= ", ";
      $details .= "$aperture";
    }
    if ($ISO != "")
    {
      /*
       * At least for Olympus, the "light value" is relative to ISO, which isn't
       * correct.
       */
      if ($light_value != 0)
      {
        $light_value += log ($ISO / 100) / log (2);
        $details .= sprintf (" (EV %2.1f)", $light_value);
      }
      $details .= ", $ISO ISO";
    }
/*    if (($saturation != "") && ($saturation != "Normal"))
      $details .= ", $saturation"; */
    $details .= "$closet";
  }
  if ($metermode != "")
  {
    $details .= "$opent" . "Meter mode:$sep$metermode";
    if ($exposure_program != "")
      $details .= " $exposure_program";
      $details .= $closet;
  }
  /* Only report exposure compensation if it exists */
  if (($EV != "") && ($EV != "0.00 EV")&& ($EV != "0.0") && ($EV != "0 EV")) /* not normal exposure, */
    $details .= "$opent" . "Exposure compensation:$sep$EV$closet";
  if ($flash != "")
  {
    $details .= "$opent" . "Flash:  $sep$flash";
    if ($flash_mode != "")
      $details .= ", $flash_mode";
    if ($manual_flash != "")
      $details .= " $manual_flash";
    if (($flash_comp != "") && ($flash_comp != "0"))
      $details .= " ($flash_comp EV)";
    $details .= "$closet";
  }
  if ($external_flash != "")
  {
    if ($flash_model)
    {
      if (($flash_model == "FL-50R")
          && ($flash_serial == "000058:1")  )
        $flash_model = "Mecablitz 58 AF-1";     /* catch this weirdness! */
      $details .= "$opent" . "Flash model:$sep$flash_model";
    }
    else
      $details .= "$opent" . "Flash model:$sep$external_flash";
    if ($external_flash_bounce != "")
      $details .= ", $external_flash_bounce";
    if ($external_flash_zoom != "")
      $details .= ", zoom $external_flash_zoom mm";
    $details .= "$closet";
  }

  if ($model != " ")
    /* Kludge: spaces before sep to line up ASCII view. */
    $details .= "$opent" . "Camera: $sep$model$closet";
  if ($lens != "")
   {
     $details .= "$opent" . "Lens:   $sep$lens";
    if (($focal_length != "") && ($equivalent_focal_length == ""))
      $details .= " at $focal_length";
    $details .= "$closet";
  }
  if ($extender != "None")
    $details .= "$opent" . "Extender:$sep$extender$closet";
  if (($equivalent_focal_length != "")
      && ($equivalent_focal_length != "0 mm") )
  {
    if ($focal_length != "")
      $details .= "$opent"
               . "Focal length:$sep$focal_length (full frame equivalent: $equivalent_focal_length mm)$closet";
    else
      $details .= "$opent" . "35 mm equivalent:$sep$equivalent_focal_length$closet";
  }
  else if ($focal_length != "")                 /* focal length but no lens info */
  {
    if ($focal_length == "0.0 mm")
    {
      $details .= "$opent" . "Focal length:$sep<i>unknown</i>";
      $focal_length = "";
    }
    else
      $details .= "$opent" . "Focal length:$sep$focal_length$closet";
  }

  if ($projection != "")
    $details .= "$opent" . "Projection:$sep$projection$closet";

  /*
   * Angles of view.  We may already have extracted it from the user comment, in
   * which case $horizontal_fov will be set.  Just print the details.  We don't
   * have a diagonal field here because it doesn't make sense in panoramas,
   * though we could calculate one if the other two angles are small enough.
   */
  if ($horizontal_fov)
    $details .= sprintf ("${opent}Field of view:$sep%3.0f$deg horizontal, %2.0f$deg vertical$closet",
                         $horizontal_fov,
                         $vertical_fov );

  /*
   * Calculate from the lens information.  We probably don't get any
   * information about the sensor scale factor, so wing it and assume that scale
   * factor < 2 means 3:2, and 2 or greater is 4:3.
   */
  else if ($scale_factor && $focal_length != "")
   {
    /* Manifest constants: 35 mm frame is 36x24 mm. */
    if ($scale_factor < 2)                      /* assume 3:2, direct ratios */
    {
      $sensor_width = 36.0 / $scale_factor;
      $sensor_height = 24.0 / $scale_factor;
    }
    else
      /*
       * Assume 4:3.  This is complicated enough that we do another kludge: we
       * start with a double-sized Four-Thirds sensor size.  Four thirds is
       * 17.3×13 mm, so that makes it 34.6×26 mm.
       */
    {
      $sensor_width = 34.6 / $scale_factor;
      $sensor_height = 26.0 / $scale_factor;
    }

    /* Diagonal of the full sensor before cropping */
    $fulldiagonal = sqrt ($sensor_width * $sensor_width + $sensor_height * $sensor_height);
    if ($sensorheight && $sensorwidth)
    {
      /*
       * We've calculated the sensor dimensions for landscape.  Check if that's
       * correct and swap if necessary.  The crop information doesn't need
       * changing.
       */
      if ($cropheight > $cropwidth)
      {
        $foo = $sensorheight;
        $sensorheight = $sensorwidth;
        $sensorwidth = $foo;
        $foo = $sensor_height;
        $sensor_height = $sensor_width;
        $sensor_width = $foo;
      }

      /* Cropped image? */
      if ($sensorheight > $cropheight)          /* cropped */
      {
        $vertcrop = $cropheight / $sensorheight;
        $sensor_height *= $vertcrop;
      }
      else if ($sensorheight < $cropheight)     /* must be panorama, and that's reduced. */
        $sensor_height = 0;                     /* cop out for now XXX */

      if ($sensorwidth > $cropwidth)          /* cropped */
      {
        $horzcrop = $cropwidth / $sensorwidth;
        $sensor_width *= $horzcrop;
      }
      else if ($sensorwidth < $cropwidth)       /* must be panorama, and that's reduced. */
        $sensor_width = 0;                     /* cop out for now XXX */
    }

    /* Do we still have dimensions? */
    if ($sensor_width && $sensor_height)
    {
      $sensor_diagonal = sqrt ($sensor_width * $sensor_width + $sensor_height * $sensor_height);
      if ($horzcrop != 1 || $vertcrop != 1)       /* cropped, report */
      {
        $details .= sprintf ("${opent}Crop factor:$sep%3.2f horizontal, %3.2f vertical$closet",
                             $horzcrop,
                             $vertcrop);
        $details .= sprintf ("${opent}Effective FL:$sep%d mm$closet",
                             $focal_length * $fulldiagonal / $sensor_diagonal );
      }
      /*
       * 114.591559 is 90 / π.
       */
      $f2 = $focal_length * 2;
      $horizontal_fov = atan ($sensor_width / $f2) * 114.591559;
      $vertical_fov = atan ($sensor_height / $f2) * 114.591559;
      $diagonal_fov = atan ($sensor_diagonal / $f2) * 114.591559;
      $details .= sprintf ("${opent}Field of view:$sep%3.1f$deg horizontal, %2.1f$deg vertical, %2.1f$deg diagonal$closet",
                           $horizontal_fov,
                           $vertical_fov,
                           $diagonal_fov );
    }
  }

  if ($format == "p")                           /* plain text, no dimensions yet */
    $details .= sprintf ("${opent}Size:   $sep%d x %d pixels$closet",
                         $cropwidth,
                         $cropheight );
  else if ($bigwidth && $bigheight)
    $details .= sprintf ("${opent}Original size:$sep%d × %d pixels, %d kB$closet",
                         $bigwidth,
                         $bigheight,
                         $bigkb );

  /* None of this is accurate enough, at least not with Olympus */
  if ($focus != "")
     $details .= "$opent" . preg_replace ("|inf|", "$inf", "Focus:  $sep$focus $depth_of_field$closet");
  return  $details;
  }

/*
 * Get the photo dimensions.  This should be replaced by a MySQL function.
 */
function getdimensions ($id)
{
  global $photo;
  global $tinywidth;
  global $tinyheight;
  global $tinykb;
  global $smallwidth;
  global $smallheight;
  global $smallkb;
  global $bigwidth;
  global $bigheight;
  global $bigkb;
  global $havedimensions;
  global $dir;
  global $path;
  global $debug;
  global $mylevel;                              /* interest level of this image */
  global $imagelevel;                           /* minimum level to display */

  $havedimensions = 0;                          /* set when we find dimensions of this photo */
  $listfile = "$path/$dir/photolist.php";
  $photos = fopen ($listfile, "r");
  if ($photos == FALSE)
  {
    print '<p><font color="red">Can\'t open ' . "$listfile: $php_errormsg" . '</font></p>';
    return;
  }

  while ($info = fscanf ($photos,
                         "%s %s %s %s %s %s %s %s %s %s %s\n",
                         $photo,
                         $tinywidth,
                         $tinyheight,
                         $tinykb,
                         $smallwidth,
                         $smallheight,
                         $smallkb,
                         $bigwidth,
                         $bigheight,
                         $bigkb,
                         $mylevel ))
  {
    if ($photo == $id)
    {
      if ($info < 11)                           /* no $mylevel specified, */
        $mylevel = 3;                           /* assume average */
      $havedimensions = 1;
      break;
    }
  }
  fclose ($photos);
}

/*
 * Work around strrangenesses in PHP substr_replace.  Put the character $c at
 * offset in $text, return the resultant string.
 */
function setimagesize ($text, $c, $offset)
{
  if ($offset > 0)
    return substr ($text, 0, $offset) . $c . substr ($text, $offset + 1);
  else
    return $c . substr ($text, $offset + 1);
}

/*
 * Generate a complete img tag.
 */
function mklinkedimage ($href, $image, $photoseq, $height, $width, $widthtxt, $compareimage, $comparesrc)
{
  global $isrss;
  global $loadimage;                            /* image to display while loading the real one */
  global $debug;
  global $isrss;

  $thisimage = basename ($image);              /* file name part in HTML format */
  if ($compareimage == "")
    $title = "Photo $thisimage";
  else
    $title = "Alternate image ". basename ($compareimage) . ".  Move cursor off image to see the original";

  if (0)                                        /* XXX */
    print <<< EOS
      <pre>
      href:  $href
      image:  $image
      photoseq:  $photoseq
      height:  $height
      width:  $width
      widthtxt:  $widthtxt
      compareimage:  $compareimage
      comparesrc:  $comparesrc
      thisimage:  $thisimage
      title:  $title
      </pre>
EOS;

  if ($isrss)                                   /* most of the fun things won't work */
  {
    print <<< EOS
  <a id="Photo-$photoseq" name="Photo-$photoseq"
          href="$href">
          <img alt="This should be $thisimage.  Is it missing?" border="0" id="Photo_$photoseq"
               title="$title.  Click to redisplay $widthtxt."
               src="$image"
               width="$width"
EOS;
    if ($height)
      print <<< EOS
 height="$height"
EOS;
    print <<<EOS

           />
          </a>

EOS;
  }
  else                                          /* HTML, let her rip */
  {

  /*
   * Note the subtle distinction between name and id of the a tag (Photo-$photoseq)
   * and the id of the img tag (Photo_$photoseq).
   */

    print <<< EOS
  <a id="Photo-$photoseq" name="Photo-$photoseq"
          href="$href">
          <img alt="This should be $thisimage.  Is it missing?" border="0" id="Photo_$photoseq"
               title="$title.  Click to redisplay $widthtxt."
               class="lazy" src="$loadimage"
               data-original="$image"
               width="$width"
EOS;
    if ($height)
      print <<< EOS
 height="$height"
EOS;
    print "
";
    if ($compareimage == "")
      print <<<EOS
               onmouseover="stm (exif_info [$photoseq], Tipstyle)"
               onmouseout="htm()"

EOS;
    else
      print <<<EOS
               onmouseover="setimage('Photo_$photoseq', '$comparesrc'); stm (exif_info [$photoseq], Tipstyle)"
               onmouseout="setimage('Photo_$photoseq', '$image'); htm()"

EOS;
    print <<<EOS
           />
          </a>
          <noscript><img src="$image" alt="Version for non-scriptable browsers"
                         width="$width" height="$height"/>
          </noscript>

EOS;
  }

}

/*
 * Generate HTML to display a single photo.
 *
 * This assumes a relatively rigid hierarchy: one directory containing three
 * subdirectories, named tiny, small and big.  The next level up in the
 * directory naming hierarchy must be the date encoded at YYYYMMDD.
 *
 * $image can be the relative pathname of an image in any of these directories.
 *
 * Variables:
 *
 * $defaultsize is the size of the image ("0" to "4").
 *
 * ${$size}width, ${$size}width, ${$size}width are the width, height and size
 * (in kB) of each image.
 *
 * $dir is the name of the directory (e.g. "/grog/Photos/20080815")
 *
 * Optional parameters:
 *
 * $href is an href link.  By default, it's to the next bigger version of this
 *    photo.
 *
 * $description is a description to include to the right of the photo.  By
 *    default it's omitted for tiny photos and is the exposure details for
 *    larger photos.
 */

function showphoto ($image/* , $compareimage */)
{
  global $photo;
  global $tinywidth;
  global $tinyheight;
  global $tinykb;
  global $smallwidth;
  global $smallheight;
  global $smallkb;
  global $bigwidth;
  global $bigheight;
  global $bigkb;
  global $havedimensions;
  global $dir;
  global $path;
  global $debug;
  global $defaultsize;                          /* 1, 2 or 3 */
  global $showinfo;                             /* override 1 and show exposure info */
  global $photodetails;
  global $photoseq;                             /* sequence number of photo on this page */
  global $imagesizes;
  global $view;
  global $servername;                           /* name of server, with http:// */
  global $edit;
  global $topics;
  global $dirdate;
  global $thisimage;                            /* name of this image */
  global $havetopics;                           /* set if we're doing topic processing */
  global $offtopic;                             /* set if we have turned this topic off. */
  global $isrss;
  global $mylevel;                              /* interest level of this image */
  global $imagelevel;                           /* interest level of the user */
  global $myurl;                                /* URL of this script */
  global $who;                                  /* top level directory name */
  global $loadimage;                            /* image to display while loading the real one */

/* Left-over old parameters */
  $href = "";                                   /* XXX do something about this */
  $description = "";                            /* XXX do something about this */
  if ($havetopics && $offtopic)                 /* not generating anything */
    return;                                     /* give up */
  $imagedirs = array ("", "tiny", "small", "big", "big"); /* directory for each image size */

  if ($edit)
    $editparm = "&amp;edit=1";                  /* parameter to pass in GETDATA */
  else
    $editparm = "";
  if ($view)
    $viewparm = "&amp;view=$view";              /* parameter to pass in GETDATA */

  else
    $viewparm = "";
  $dir = dirname ($image);                      /* remove the last part of the directory */
  $dirdate = basename ($dir);                   /* just the date component */
  $thisimage = basename ($image);               /* file name part */
  $photoseq++;                                  /* make sure we have a unique name for each photo */
  $tagid = "Photo-$photoseq";                   /* this one for tags */
  $docroot = $_SERVER ["DOCUMENT_ROOT"];
  /*
   * This is (still) a mess.  We need to extract the following information:
   *
   * 1: The system-related path name to the image.  This is $path.
   * 2: The web server relative URL of the name we have supplied.  This is
   *    $tlpath.
   *
   * Things are made worse by the fact that the $image parameter can be relative
   * to the current user, or it can be "absolute" in the sense that it refers to
   * the document root.
   */
  if (substr ($image, 0, 1) == "/")             /* "absolute" path name */
  {
    $tlpath = "";                               /* top level URL path */
    $paths = explode ("/", $image);             /* get user name from image */
    $user = "/$paths[1]/";                      /* user name */
    $path = "$docroot";                         /* but we go from doc root */
  }
  else
  {
    $paths = explode ("/", $_SERVER ["SCRIPT_NAME"]);
    $path = "$docroot/$paths[1]";
    $tlpath = "/$paths[1]/";                    /* top level URL path */
    $user = $tlpath;                            /* which is also user name */
  }

  if (! array_key_exists ($tagid, $photodetails))
  {
    $photodetails [$tagid] = array ("seq" => $photoseq);
    if (($photodetails [$tagid] ["size"] = substr ($imagesizes, $photoseq, 1)) == "")
    {
      $photodetails [$tagid] ["size"] = $defaultsize;
      $imagesizes .= $defaultsize;
    }
  }

  $imagesize = $photodetails [$tagid] ["size"];

  getdimensions ($thisimage);                   /* Get the image dimensions */
  /* XXX Decide whether to display this image.  This needs fixing: we need to
   * count the links, and we need to pass the $imagelevel in all self-referent
   * link URLs.
   */
  if ($debug)                                        /* XXX */
    print <<< EOS
    <p>
    image level: $imagelevel
    my level: $mylevel
    </p>
EOS;
  if ($mylevel < $imagelevel)
    return;
  $photodate =  formatdate ("l, j F Y", validdate ($dirdate, "date"));
  $basephoto = preg_replace ("|\.[^.]*$|", "", basename ($photo));
  $phototitle = str_replace ("-", " ", $basephoto);
  $exif_file = "$tlpath$dir/$basephoto.exif";
  if ($debug)                                        /* XXX */
  print <<< EOS
    <pre>
    image level: $imagelevel
    my level: $mylevel
      exif_file: $exif_file
    </pre>
EOS;

  /*
   * Decide how to display the image.  If it's tiny, we just display it, unless
   * $showinfo is set.  Otherwise we put it in a table with a description.  The
   * location of the description depends on the width of the image, not its
   * "size".  Empirically we choose a $maxwidth of 800 pixels; if the image is
   * narrower than that, put the description on the right.  Otherwise put it
   * below.
   */

  $maxwidth = 800;                              /* magic width */
  switch  ($imagesize)
  {
  case 3:                                       /* big */
  case 4:                                       /* very big */
    $mywidth = $bigwidth;
    $myheight = $bigheight;
    $mykb = $bigkb;
    break;

  case 2:                                       /* small */
    $mywidth = $smallwidth;
    $myheight = $smallheight;
    $mykb = $smallkb;
    break;
  case 0:                                       /* invisible */
    /*
     * We don't need this for "invisible", but it simplifies a test below.
     */
  case 1:                                       /* tiny */
    $mywidth = $tinywidth;
    $myheight = $tinyheight;
    $mykb = $tinykb;
  }

  /* Pass on diary topics if they're there. */
  if ($topics != "")
    $topicstr = "&amp;topics=$topics";
  else
    $topicstr = "";
  $imagedir = $imagedirs [$imagesize];
  $dir = dirname ($image);                      /* remove the last part of the directory */
  $dirdate = basename ($dir);                   /* just the date component */
  /* XXX remove thisimage */
  $thisimage = htmlify (basename ($image));     /* file name part in HTML format */
  $imagesrc = "$servername$tlpath$dir/$imagedir/$thisimage"; /* fully qualified name */
  $localimagepath = "$path/$dir/$imagedir/" . basename ($image); /* and local pathname */

  if ($debug)
  {
    print "<pre>path $path
image $image
tlpath $tlpath
dir $dir
user $user
tagid $tagid
defaultsize $defaultsize
photo $photo
basephoto $basephoto
imagesrc $imagesrc
localimagepath $localimagepath
docroot $docroot
imagesize $imagesize
";
  print_r ($photodetails [$tagid]);
  print_r ($photodetails);
  print "imagesizes: " . $imagesizes;
  print "</pre>";
  }

  if ($imagesize && ! file_exists ($localimagepath)) /* file missing? */
  {
    /*
     * XXX Horrible kludge.  Rename old cached images.
     */
    $altimagepath = preg_replace (":globosa:", "weyeriana", $localimagepath);
    if (file_exists ($altimagepath))
    {
      $altimagesrc = preg_replace (":globosa:", "weyeriana", $imagesrc);
      print <<< EOS
      <p style="color:red">
      This page appears to be an old cached version.  It refers to $imagesrc, which
      has been replaced with $altimagesrc.  Please update your cache.
       </p>

EOS;
      $localimagepath = $altimagepath;
      $thisimage =  preg_replace (":globosa:", "weyeriana", $thisimage);
      $imagesrc = $altimagesrc;
      $phototitle = str_replace ("-", " ", basename ($imagesrc, ".jpeg"));
    }
    else
    {
      if (islocal ())
        print <<< EOS
      <p style="color:red">
        There should be an image <a href="$imagesrc">$imagesrc</a> here, but it is not present on
        this server.  The webmaster has been alerted.  Come back soon.
       </p>

EOS;
      $imagesrc = "$servername$tlpath$dir/$imagedir/" . basename ($image); /* Remove HTMLification */
      $URL = $_SERVER ["SERVER_NAME"] . $_SERVER ["REQUEST_URI"];
      $REMOTE_HOST = $_SERVER["REMOTE_HOST"];
      $REMOTE_ADDR = $_SERVER["REMOTE_ADDR"];
      mail ($_SERVER ["SERVER_ADMIN"],
          "Missing image:  $imagesrc ($localimagepath)",
          <<< EOS
Page URL:       http://$URL
Image URL:      $imagesrc
File name:      $localimagepath

Remote host:    $REMOTE_HOST
Remote IP:      $REMOTE_ADDR

EOS
          );

      return;
    }

  }

  /* Do we have a compare image?  If so, the same again, please. */
  if (! $isrss && func_num_args () > 1)         /* we have an image to compare */
  {
    $compareimage = func_get_arg (1);
    $comparedir = dirname ($compareimage);                      /* remove the last part of the directory */
    $comparedirdate = basename ($comparedir);                   /* just the date component */
    $comparethisimage = htmlify (basename ($compareimage));     /* file name part in HTML format */
    $compareimagesrc = "$servername$tlpath$dir/$imagedir/$comparethisimage"; /* fully qualified name */
    $comparelocalimagepath = "$path/$comparedir/$imagedir/" . basename ($compareimage); /* and local pathname */
    if ($imagesize && ! file_exists ($comparelocalimagepath)) /* file missing? */
    {
      $compareimagesrc = "$servername$tlpath$dir/$comparedir/" . basename ($compareimage); /* Remove HTMLification */
      $URL = $_SERVER ["SERVER_NAME"] . $_SERVER ["REQUEST_URI"];
      $REMOTE_HOST = $_SERVER["REMOTE_HOST"];
      $REMOTE_ADDR = $_SERVER["REMOTE_ADDR"];
      mail ($_SERVER ["SERVER_ADMIN"],
            "Missing comparison image:  $compareimagesrc ($comparelocalimagepath)",
          <<< EOS
Page URL:       http://$URL
Image URL:      $compareimagesrc
File name:      $comparelocalimagepath
Main image:     $imagesrc

Remote host:    $REMOTE_HOST
Remote IP:      $REMOTE_ADDR

Debug:
compareimage   $compareimage
comparedir     $comparedir
comparedirdate   $comparedirdate
comparelocalimagepath   $comparelocalimagepath
dir   $dir

EOS
            );

      /*
       * If we don't have a compare image, just proceed as if none had been
       * specified.
       */
      $compareimage = "";
      $title = "Photo $thisimage";
    }
    else
      $title = "Alternate image ". basename ($compareimage) . ".  Move cursor off image to see the original";
  }
  else
  {
    $compareimage = "";
    /* XXX remove $title here when changes finished */
    $title = "Photo $thisimage";
  }

  if ($debug)                                   /* XXX */
  {
    print "<pre>";
    print_r ($_SERVER);
    print "Description: $description</br>";
    print "compareimage: $compareimage</br>";
    print "<p>dir: $dir</p>";
    print "<p>user: $user</p>";
    print "<p>servername: $servername</p>";
    print "</pre>";
  }
  if ($compareimage != "")
  {
    $comparedir = dirname ($compareimage);      /* remove the last part of the directory */
    $comparethisimage = htmlify (basename ($compareimage)); /* file name part in HTML format */
    $comparesrc = "$servername$tlpath$comparedir/$imagedir/$comparethisimage"; /* fully qualified name */
  }
  else
    $comparesrc = "";

  $makemeinvisible = setimagesize ($imagesizes, "0", $photoseq);
  $makemetiny = setimagesize ($imagesizes, "1", $photoseq);
  $makemesmall = setimagesize ($imagesizes, "2", $photoseq);

  if (($imagesize == 3) || (($mywidth > $maxwidth))
      && (($imagesize > 1) || $showinfo))
  {
    /* Wide  image.  Put the description below. */
    /* First line starts indented */
    print <<< EOS
    <br />
    <a id="$tagid" name="$tagid" />

      <!-- Display wider $thisimage -->
      <!-- Work around rendering engine breakage.  Without this &nbsp;, a number of browsers will
           place small images to the right of a big image (even though the table is 100% wide).  I
           haven't seen it happen here, but I don't trust HTML further than I can throw it.  If this
           is my bug, I would appreciate feedback about what (grog, 19 January 2009).
        -->
      &nbsp;

EOS;

  /*
   * XXX References to $href here are leftovers from when we handled parameters.
   * Currently it's never set at this point, but that might change.
   *
   * In the following, mouseover depends on whether we have a second image to
   * compare or not.  If we do, we display it on mouseover.  Otherwise we show
   * the EXIF data.
   */
  switch ($imagesize)
      {
    case 2:                                     /* Small image */
      if ($href == "")
        $href = "$myurl"
          . displayargs ($defaultsize, setimagesize ($imagesizes, "3", $photoseq))
          . "#$tagid";

      mklinkedimage ($href, $imagesrc, $photoseq, $myheight, $mywidth,
                     "image at screen width ($bigkb kB, $bigwidth x $bigheight pixels)",
                     $compareimage, $comparesrc);
      break;

    case 3:                                     /* "Big" image at screen size */
      if ($href == "")
        $href = "$myurl" . displayargs ($defaultsize, setimagesize ($imagesizes, "4", $photoseq)) . "#$tagid";

      mklinkedimage ($href, $imagesrc, $photoseq, 0, "100%",
                     "full size image ($bigwidth x $bigheight, $bigkb kB)",
                      $compareimage, $comparesrc);
      break;

    case 4:                                     /* full size */
      if ($href == "")
        $href = "$myurl" . displayargs ($defaultsize, setimagesize ($imagesizes, "2", $photoseq)) . "#$tagid";

      mklinkedimage ($href, $imagesrc, $photoseq, $myheight, $mywidth,
                     "small version of image",
                      $compareimage, $comparesrc);
      }
    $myargs = displayargs ("3");
    $sizename = $imagedirs [$imagesize];
    print <<< EOS
      <table width="100%" summary="Image info" align="left">  <!--  wide $sizename image details, links -->
        <tr valign="top">
          <td>
            <table summary="Image name and size, links">
              <tr>
                <td>
                  Image title:
                </td>
                <td>
                  $phototitle
		  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                </td>
                <td>
                  Dimensions:
		  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                </td>
                <td>
                  $mywidth x $myheight, $mykb kB
                </td>
              </tr>

              <tr>
                <td colspan="2">
                  <a href="$servername/$who/photos/Onephoto.php?image=$image&amp;size=3">
                    Make a single page with this image
                 </a>
                </td>

EOS;
    $myargs = displayargs ($defaultsize, $makemeinvisible);
    print <<< EOS
                <td colspan="2">
                  <a href="$myurl$myargs#$tagid">
                   Hide this image
                 </a>
                </td>
              </tr>
EOS;

    $myargs = displayargs ($defaultsize, $makemetiny);
    print <<< EOS

              <tr>
                <td colspan="2">
                  <a href="$myurl$myargs#$tagid">
                   Make this image a thumbnail
                 </a>
                </td>

EOS;
    $myargs = displayargs (1, "");
    print <<< EOS
                <td colspan="2">
                  <a href="$myurl$myargs#$tagid">
                    Make thumbnails of all images on this page
                  </a>
                </td>
              </tr>

              <tr>
                <td colspan="2">
EOS;

    /* If this doesn't apply, we leave an empty space */
      if ($imagesize > 2)
      {
      $myargs = displayargs ($defaultsize, $makemesmall);
    print <<< EOS
                  <a href="$myurl$myargs#$tagid">
                    Make this image small again
                 </a>

EOS;
    }

    $myargs = displayargs (2, "");
    print <<< EOS
                </td>
                <td colspan="2">
                  <a href="$myurl$myargs#$tagid">
                    Display small version of all images on this page
                  </a>
                </td>
              </tr>
              <tr>
                <td colspan="2">
EOS;
    $myargs = displayargs (1, "", $dirdate, 1);
    print <<< EOS
                  <a href="$servername{$user}photos/Photos.php$myargs">All images
                  taken on $photodate, thumbnails</a>
                  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                </td>

                <td colspan="2">
EOS;
    $myargs = displayargs (2, "", $dirdate);
    print <<< EOS
                  <a href="$servername{$user}photos/Photos.php$myargs">All images
                  taken on $photodate, small
                  </a>
                </td>
              </tr>

EOS;

      $diary = diarylink ($user, $dirdate);
      if ($diary != "")
        print <<< EOS

              <tr>
                <td colspan="2">
                  <a href="$servername$diary">Diary entry for $photodate</a>
                </td>
EOS;
    if (file_exists ("$docroot$exif_file"))
      print <<< EOS
                <td>
                  <a href="$servername$exif_file">
                    Complete exposure details
                  </a>
                </td>

EOS;
        print <<< EOS
              </tr>
            </table>                            <!-- big image details, links -->
          </td>

EOS;
    print <<< EOS
        </tr>
      </table>
      <br />
      &nbsp;
EOS;
  }

  /*
   * We get here if the image is not full width and less than $maxpixels wide.
   * XXX check this: it implies that it's only for size 2.
   */
  else if (($imagesize > 1) || $showinfo)
  {
    /* We want info displayed.  Display it to the right of the photo. */
    /* Display image with information at right */
    if ($imagesize == 1)
      $href = "$myurl" . displayargs ($defaultsize, setimagesize ($imagesizes, "2", $photoseq)) . "#$tagid";
    else                                        /* small image */
      $href = "$myurl" . displayargs ($defaultsize, setimagesize ($imagesizes, "3", $photoseq)) . "#$tagid";
    print <<< EOS
      <!-- Display $imagesize $thisimage -->
      <!-- Work around rendering engine breakage.  Without this &nbsp;, a number of browsers will
           place this image to the right of a big image (even though the table is  100% wide).  If
           this is my bug, I&#39;d appreciate feedback about what it is (grog, 19 January 2009).
        -->
      &nbsp;

      <table summary="Image and details">
        <tr>
          <td>

EOS;

        mklinkedimage ($href, $imagesrc, $photoseq, $myheight, $mywidth,
                       "image at screen width",
                       $compareimage, $comparesrc);

    print <<< EOS
          </td>

EOS;
    if ($description != "")
      print "
          <td>
           $description          </td>";        /* description ends with a \n */
    else
    {
    print <<< EOF
          <td valign="top">
            <table summary="Image details">
              <tr>
                <td>Image title: $phototitle</td>
              </tr>

EOF;
    if (file_exists ("$docroot$exif_file"))
      {
/*        print exposuredetails ("$docroot$exif_file", "c"); */
        print '
              <tr>
                <td><a href="' . $exif_file . '">Complete exposure
                details</a></td>
              </tr>';
      }
    print <<< EOF
              <tr>
                <td>Dimensions: $mywidth x $myheight, $mykb kB</td>
              </tr>
              <tr>
                <td>Dimensions of original: $bigwidth x $bigheight, $bigkb kB</td>
              </tr>

EOF;
    if ($imagesize > 1)                                          /* these don't fit beside a tiny image */
    {
      print <<<EOS

              <tr>
                <td colspan="2">
                Display this image:
                <br />
EOS;
      $myargs = displayargs ($defaultsize, $makemetiny);
      print <<< EOS

                  <a href="$myurl$myargs#$tagid">thumbnail&nbsp;</a>&nbsp;&nbsp;
EOS;
      $myargs = displayargs ($defaultsize, $makemeinvisible);
      $imagehtml = htmlify ($image, 1);
      print <<< EOS

                  <a href="$myurl$myargs#$tagid">hidden</a>&nbsp;&nbsp;
                  <a href="$servername/$who/photos/Onephoto.php?image=$imagehtml&amp;size=$imagesize">alone on  page</a>

EOS;
      if ($imagesize == 3)
      {
        $this_size = "small";
        print <<<EOS

EOS;
      $myargs = displayargs ($defaultsize, $makemesmall);
      print <<< EOS
                  <a href="$myurl$myargs#$tagid">
                    Small again
                 </a>

EOS;
      }
      else
        $this_size = "this size";
        print <<<EOS
                </td>
              </tr>

              <tr>
                <td>
                </td>
              </tr>
EOS;

      $myargs = displayargs (1, "");
      print <<<EOS

              <tr>
                <td>
                  Display all images on this page as:
                  <br />
                  <a href="$myurl$myargs#$tagid">
                  thumbnails
                  </a>
                   &nbsp;&nbsp;
EOS;

      $myargs = displayargs (2, "");
      print <<<EOS
                  <a href="$myurl$myargs#$tagid">
                  $this_size
                  </a>
                </td>
              </tr>

              <tr>
                <td>
                  Show for $photodate:
                  <br />
EOS;

      $myargs = displayargs (1, "", $dirdate, 1);
      print <<<EOS
                  <a href="$servername{$user}photos/Photos.php$myargs">thumbnails</a>
                   &nbsp;&nbsp;
EOS;

      $myargs = displayargs (2, "", $dirdate, 1);
      print <<<EOS
                  <a href="$servername{$user}photos/Photos.php$myargs">small images</a>
EOS;

      $diary = diarylink ($user, $dirdate);
      if ($diary != "")
        print <<< EOS

                   &nbsp;&nbsp;
      <a href="$servername$diary">diary entry</a>
                </td>
              </tr>

EOS;
      }
    print <<<EOS
            </table>
          </td>

EOS;
    }

    print <<<EOS
        </tr>
      </table>

EOS;
  }
  else if ($imagesize == 1)                     /* tiny image with no info */
  {
    if ($debug)                                      /* XXX debug */
    {
      print "<p> imagesizes: $imagesizes</p>";
      print "<p> photoseq: $photoseq</p>";
      print "<p> image: $image</p>";
      print "<p> imagesizes+: " . setimagesize ($imagesizes, "2", $photoseq) . "</p>";
    }
    if ($href == "")
      $href = "$myurl" . displayargs ($defaultsize, setimagesize ($imagesizes, "2", $photoseq)) . "#$tagid";
/*    $href="$myurl?size=$defaultsize$topicstr&amp;imagesizes=" . setimagesize ($imagesizes, "2", $photoseq)
      . "&amp;dirdate=$dirdate$editparm$viewparm&amp;image=$thisimage#$tagid"; */

      mklinkedimage ($href, $imagesrc, $photoseq, $myheight, $mywidth,
                     "larger version of image",
                      $compareimage, $comparesrc);

  }

  /* Finally, print EXIF text for popup. */
  if (! $isrss && file_exists ("$docroot$exif_file"))
  {
    $exifdata = exposuredetails ("$docroot$exif_file", "j");
    if ("$compareimage" != "")
    {
      $basephoto = preg_replace ("|\.[^.]*$|", "", basename ($photo));
      $compareexif_file = "$tlpath$comparedir/"
                          . preg_replace ("|\.[^.]*$|", ".exif", basename ($compareimage));
      if (file_exists ("$docroot$compareexif_file"))
        $exifdata .= "<tr><td>&nbsp;</td></tr>"
                  . "<tr><td>Comparison:</td><td>$comparethisimage</td></tr>"
                  . exposuredetails ("$docroot$compareexif_file", "j");
    }

    if ($exifdata != "")
    {
      if ($imagesize == 1)                      /* instructions only for tiny images */
        print <<< EOS
          <script type="text/javascript">
             <!--
             exif_info [$photoseq] = ["Image $thisimage<br />Click on image for up to 3 larger versions",
                              "$exifdata"];
             // -->
          </script>

EOS;
      else
        print <<< EOS
          <script type="text/javascript">
             <!--
             exif_info [$photoseq] = ["Image $thisimage",
                              "$exifdata"];
             // -->
          </script>

EOS;
    }
  }
}

/*
 * Show a photo with a title underneath.
 */
function showphoto_with_title ($image, $title)
{
  global $defaultsize;

  $dir = dirname ($image);                      /* remove the last part of the directory */
  $dirdate = basename ($dir);                   /* just the date component */
  print <<< EOS
      <table>
        <tr>
          <td>

EOS;
  showphoto ($image);
  print <<< EOS
          </td>
	</tr>
	<tr>
	  <td>
	    $title
	  </td>
	</tr>
      </table>

EOS;
}

/*
 * Show a photo and put a date underneath if this is a tiny image
 * (the others already show the date).
 *
 * XXX this doesn't include code for selective sizes.
 */
function showphoto_with_date ($image)
{
  global $defaultsize;

  showphoto ($image);

  $dir = dirname ($image);                      /* remove the last part of the directory */
  $dirdate = basename ($dir);                   /* just the date component */
  if ($defaultsize == 1)
  {
    $photodate =  formatdate ("l, j F Y", validdate ($dirdate, "date"));
    showphoto_with_title ($image, $photodate);
  }
  else
    showphoto ($image);
}

/*
 * Show a button to select $showdetails.
 */
function showdetailbutton (/* int $section */)
{
  global $showdetails;
  global $showdetail_button;
  global $size;
  global $imagesizes;
  global $image;
  global $showdetails;
  global $topics;
  global $view;
  global $edit;

  if (func_num_args () > 0)
    $section = func_get_arg (0);
  else
    $section = 0;
  if ($section)
  {
    print <<< EOS
      <table>
        <tr>
          <td>

EOS;
  }

  $showdetail_button++;
  $me = $_SERVER ["SCRIPT_NAME"] . displayargs () . "#B-$showdetail_button";
  if ($showdetails == "")                       /* not set, */
  {
    $show = "show";
    $text = "Show more details";
  }
  else
  {
    $show = "";
    $text = "Show fewer details";
  }

  print <<< EOS
      <a name="B-$showdetail_button" id="B-$showdetail_button" />
        <form action="$me" method="get">
          <input type="hidden" name="imagesizes" value="$imagesizes" />
          <input type="hidden" name="image" value="$image" />
          <input type="hidden" name="showdetails" value="$showdetails" />
          <input type="hidden" name="topics" value="$topics" />
          <input type="hidden" name="view" value="$view" />
          <input type="hidden" name="showdetails" value="$show" />
          <input type="submit" value="$text" />
       </form>

EOS;
  if ($section)
  {
    print <<< EOS
          </td>
          <td>
            <a name="B-$showdetail_button" id="B-$showdetail_button" />
              <form action="$me" method="get">
<!--      <input type="hidden" name="size" value="$size" /> -->
          <input type="hidden" name="imagesizes" value="$imagesizes" />
          <input type="hidden" name="image" value="$image" />
          <input type="hidden" name="showdetails" value="$showdetails" />
          <input type="hidden" name="topics" value="$topics" />
          <input type="hidden" name="view" value="$view" />
                <input type="hidden" name="showdetails" value="$show" />
                <input type="submit" value="$text" />
              </form>
          </td>
        </tr>
      </table>

EOS;
  }
  else
    print "      <br />\n";
}

/*
 * Conditional showphoto (): only show photo if thee global variable
 * $showdetails is set.
 */
function showphoto_detail ($image/* , $href, $description */)
{
  global $showdetails;
  global $photoseq;
  global $photodetails;
  global $imagesizes;
  global $defaultsize;

  if ($showdetails != "")
    showphoto ($image);
  else                                          /* mark our place */
  {
    $id = basename ($image);                    /* file name part */
    $photoseq++;                                /* make sure we have a unique name for each photo */
    $tagid = "P-" . basename ($id, ".jpeg") . "-$photoseq"; /* this one for tags */
    print "<a name=\"$tagid\" id=\"$tagid\" />\n";
    if (! array_key_exists ($tagid, $photodetails))
    {
      $photodetails [$tagid] = array ("seq" => $photoseq);
      if (($photodetails [$tagid] ["size"] = substr ($imagesizes, $photoseq, 1)) == "")
      {
        $photodetails [$tagid] ["size"] = $defaultsize;
        $imagesizes .= $defaultsize;
      }
    }
  }
}

/*
 * Show an image with a link to a single larger image.  This is typically the
 * case for graphs such as the satellite statistics.  We can't rely on a
 * relationship between the image names, so we provide both URLs.
 * $height and $width are height and width of the small image.
 * $desc is a description (alt=) for the image.
 */
function imagepair ($small, $big, $width, $height, $desc)
{
  /* Make sure URLs are fully qualified */
  $small = complete_url ($small);
  $big = complete_url ($big);
  print <<< EOS
<a href="$big">
          <img alt="$desc" title="$desc: click on image for larger version."
            src="$small"
EOS;
  if (($height != 0) && ($width != 0))
    print <<< EOS
            height="$height" width="$width"
EOS;
  print <<< EOS
          />
        </a>

EOS;
}

/*
 * General purpose image display.
 * If present, $desc is a description (alt=) for the image.
 * $height and $width may be omitted.
 * If $height or $width are present, the image is displayed at that size with a
 * link to the original unless $nolink is set.
 */
function oneimage ($src/*, $desc, $width, $height, $nolink */)
{
  global $loadimage;
  global $isrss;

  /* Make sure URLs are fully qualified */
  $src = complete_url ($src);
  $width = "";                                  /* no width specified */
  $height = "";                                 /* no height specified */
  $nolink = 0;                                  /* we want to link to original image */
  $desc = "Image";                              /* to make validators happy */
  switch  (func_num_args ())
  {
  case 5:
    $nolink = 1;
      /* FALLTHROUGH */
  case 4:
      $height = func_get_arg (3);
      /* FALLTHROUGH */
  case 3:
      $width = func_get_arg (2);
      /* FALLTHROUGH */
  case 2:
    $desc = func_get_arg (1);
    break;
  }
  $src = complete_url ($src);
  if (($height == "") && ($width == ""))        /* display at original size */
    {
    if ($isrss)
      print <<< EOS
<img alt="$desc"
          title="$desc"
          src="$src"
        />

EOS;
   else
      print <<< EOS
<img alt="$desc"
          title="$desc"
          src="$loadimage"
          data-original="$src"
        />
     <noscript>
       <img src="$src"
            alt="Version for non-scriptable browsers"/>
     </noscript>

EOS;
    }
  else if ($nolink)                             /* just this image */
  {
    if ($isrss)
      print <<< EOS
<img alt="$desc"
          title="$desc"
          src="$src"
        />

EOS;
    else
     print <<< EOS
<img alt="$desc"
          title="$desc"
          src="$loadimage"
          data-original="$src"

EOS;
    if ($height != "")
      print <<< EOS
          height="$height"

EOS;
    if ($width != "")
      print <<< EOS
          width="$width"

EOS;
    print <<< EOS
          />
    }
    if (! $isrss)
       <noscript>
         <img src="$src"
              alt="Version for non-scriptable browsers"
              width="$width" height="$height" />
       </noscript>

EOS;
  }
  else                                          /* we have scaling, want a link */
  {
    if ($isrss)
      print <<< EOS
<a href="$src">
      <img alt="$desc"
          title="$desc.  Click to see in original size."
          src="$src"
        />

EOS;
    else
     print <<< EOS
<a href="$src">
      <img alt="$desc"
          title="$desc.  Click to see in original size."
          src="$loadimage"
          data-original="$src"

EOS;

    if ($height != "")
      print <<< EOS
          height="$height"

EOS;
    if ($width != "")
      print <<< EOS
          width="$width"

EOS;
    print <<< EOS
          />
        </a>
EOS;
    if (! $isrss)
    print <<< EOS

       <noscript>
         <img src="$src"
              alt="Version for non-scriptable browsers"
              width="$width" height="$height" />
       </noscript>

EOS;
  }
}

/*
 * Create an img tag with mouseover if we can do it (we can't with RSS).
 * $id:   id
 * $off:  image to display without mouse cursor.
 * $on:   image to display with mouse cursor.
 */
function mouseoverimage ($id, $on, $off)
{
  global $isrss;

  $off = complete_url ($off);
  $on = complete_url ($on);
  $thisimage = basename ($off);

  print <<< EOS

        <img alt="This should be $thisimage.  Is it missing?" border="0"
	   id="$id"

EOS;
  if (! $isrss)
    print "
           onmouseover=\"setimage('$id',
                                 '$on');\"
           onmouseout=\"setimage('$id',
                                '$off');\"
";
  print <<< EOS
           title="Before image"
	   src="$off" />
EOS;
}

/*
 * Get the description string for the photos dated $dirdate.
 * This needs to be replaced by a database lookup.
 *
 * This is a kludge version that assumes that dirlist is in the same directory
 * as the photos, and that it's cwd.  Both are currently the case.
 *
 * This was taken from similar code in photos/photos.php.
 */

function photo_description ($dirdate)
{
  $description = array ();
  /* dirlist has the format 'directory name', 'description' */
  $photos = fopen ("dirlist", "r");
  while ($line = fgets ($photos))
  {
    $info = explode (" ", $line, 2);            /* make two parts */
    /* extract the parameters from the photo list */
    $date = $info [0];

    if ($date < $dirdate)                       /* not there yet */
      $description ["prev"] = $info;
    else if ($date == $dirdate)                 /* found it */
      $description ["current"] = $info;
    else                                        /* must be past it */
    {
      fclose ($photos);
      $description ["next"] = $info;
      return $description;
    }
  }
  fclose ($photos);
  return $description;
}

/*
 * Display an image and play a "movie" in Canon's format.
 */
function canon_movie ($path)
{
  print "<a href=\"$path.AVI\">";
  oneimage ("$path.THM");
  print "</a>";
}

/*
 * Display a flash video.
 * Not sure where we can use this; see youtube() for something functional.
 */
function video ($url/* , $width, $height */)
{
  global $servername;                           /* name of server, with http:// */

  $width = 320;
  $height = 240;
  switch  (func_num_args ())
  {
  case 3:
    $height = func_get_arg (2);
    /* FALLTHROUGH */
  case 2:
    $width = func_get_arg (1);
    /* FALLTHROUGH */
  default:
    break;
  }
  print <<< EOS
<iframe width="$width"
              height="$height"
              src="$url"
              frameborder="0">
      </iframe>

EOS;
}

/* <object type="application/x-shockwave-flash"
 *              data="$url"
 *              width="$width" height="$height">
 *      <param name="movie"
 *             value="$url" />
 *      <img src="$servername/grog/no-flash.gif"
 *           width="292" height="127" alt="" />
 *      </object>
 */

/*
 * Display a Youtube video.  This takes just the code part.
 */
function youtube ($id/* , $width, $height */)
{
  global $isrss;

  $width = 320;
  $height = 240;
  switch  (func_num_args ())
  {
  case 3:
    $height = func_get_arg (2);
    /* FALLTHROUGH */
  case 2:
    $width = func_get_arg (1);
    /* FALLTHROUGH */
  default:
    break;
  }
  if ($isrss)                                   /* we can't include iframes in RSS */
    print <<< EOS
<p>
      <a href="http://www.youtube.com/watch?v=$id"><img src="http://i.ytimg.com/vi/$id/default.jpg" /></a>
      </p>

EOS;
  else
    print <<< EOS
<iframe width="$width"
            height="$height"
            src="http://www.youtube.com/embed/$id"
            frameborder="0">
     </iframe>

EOS;
}
