<?php  /* Hey, emacs!  Edit this file in -*- php -*- mode! */

/*
 * $Id: utils.php,v 1.71 2012/07/10 23:58:56 grog Exp $
 * Make a single row of a table entry.
 * This should be static; it's called only from makerows ().
 * $alignments is l, c or r for left, centre or right.
 * $style, if present, is a style tag such as b or i or strong.
 */
function makerow ($row, $alignments, $styles)
{
  $row = preg_replace ("|   *|", "\t", $row);     /* convert 2 or more spaces to tabs */
  $row = preg_replace ("|\t\t+|", "\t", $row);    /* convert 2 or more tabs to single tab */
  $cols = explode ("\t", $row);
  $colnum = 0;                                  /* column number, starting at 0 */

  print <<< EOS

       <tr>

EOS;
  foreach ($cols as $col)
  {
    /*
     * Decide what alignment we will use for this column.  We always have at
     * least one char in alignments, so this test allows us to propagate the
     * last character if we run out.
     */
    if ($alignments [0])
    {
      switch ($alignments [0])
      {
      case 'l':
        $alignment = "left";
        break;
      case 'c':
        $alignment = "center";
        break;
      case 'r':
        $alignment = "right";
        break;
      }
      $alignments = substr ($alignments, 1);    /* shift left */
    }
    /* Leave a space between columns */
    if ($colnum)                                /* not first column */
      print <<< EOS
          <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>

EOS;
    if ($col == "-")                            /* placeholder */
      $col = "";                                /* remove it */
    else if (substr ($col, 0, 1) == "@")
    {
      $date = validdate (substr ($col, 1), "date");
      $col = formatdate ("j F Y", $date);
    }
    else if (substr ($col, 0, 1) == "^")        /* email address */
    {
      $realcol = substr ($col, 1);
      $col = "<a href=\"mailto:$realcol\">$realcol</a>";
    }
    if (array_key_exists ($colnum, $styles))    /* we want a specific format style */
    {
      $style = $styles [$colnum];               /* get it */
print <<< EOS
          <td valign="top" align="$alignment"><$style>$col</$style></td>

EOS;
    }
      else
    print <<< EOS
          <td valign="top" align="$alignment">$col</td>

EOS;
    $colnum++;
  }
  print <<< EOS
       </tr>

EOS;
}

/*
 * Convert a \n delimited string into multiple table rows.
 *
 * This should be surrounded by a <table> tag in the source.
 *
 * $alignments, if present, is a set of letters specifying the alignment of each
 * column (l, c or r for left, centre and right.
 */
function makerows ($glup                                                             /* , $alignments, $styles */)
{
  if (func_num_args () > 1)
  {
    $alignments = func_get_arg (1);
    if (func_num_args () > 2)
    {
      $styles = func_get_arg (2);
      $styles = explode (' ', $styles);
    }
    else
      $styles = array ();
  }
  else
  {
    /* This is the historical value: first col right, the rest left. */
     $alignments = "rl";
     $styles = array ();
  }

  $rows = explode ("\n", $glup);
  $i = 0;
  foreach ($rows as $row)
    makerow ($row, $alignments, $styles);
}

/*
 * Variant of makerows.
 * Convert a \n delimited string into multiple table rows.
 * Add the contents of column $col, present subtotals on an empty input line and
 * a total at the end.
 *
 * This should be surrounded by a <table> tag in the source.
 *
 * $alignments, if present, is a set of letters specifying the alignment of each
 * column (l, c or r for left, centre and right.
 */
function addrows ($glup, $col /* , $alignments, $styles */)
{
  $sum = 0;
  $partsum = 0;
  if (func_num_args () > 2)
  {
    $alignments = func_get_arg (2);
    if (func_num_args () > 3)
    {
      $styles = func_get_arg (3);
      $styles = explode (' ', $styles);
    }
    else
      $styles = array ();
  }
  else
  {
    /* This is the historical value: first col right, the rest left. */
     $alignments = "llr";
     $styles = array ();
  }

  $rows = explode ("\n", $glup);
  $i = 0;
  foreach ($rows as $row)
    {
    makerow ($row, $alignments, $styles);
    $row = preg_replace ("|   *|", "\t", $row);     /* convert 2 or more spaces to tabs */
    $row = preg_replace ("|\t\t+|", "\t", $row);    /* convert 2 or more tabs to single tab */
    $cols = explode ("\t", $row);
    if (array_key_exists ($col, $cols))
      $partsum += $cols [$col];
    else if (! array_key_exists (1, $cols))
      {
      if ($col > 0)
        {
        $foo = "Subtotal:  ";
        for ($i = 1; $i < $col; $i++)
          $foo .= "-  ";
        }
      else
        $foo = "";
      $foo .= $partsum;
      makerow ($foo, $alignments, $styles);    /* partial sum */
      makerow ("", $alignments, $styles);    /* partial sum */
      makerow ("", $alignments, $styles);    /* partial sum */
      makerow ("", $alignments, $styles);    /* partial sum */
      makerow ("", $alignments, $styles);    /* partial sum */
      $sum += $partsum;
      $partsum = 0;
      }
    }
    $sum += $partsum;
    if ($col > 0)
      {
      $foo = "Total:  ";
      for ($i = 1; $i < $col; $i++)
        $foo .= "-  ";
      }
    else
      $foo = "";
    $foo .= $sum;
    makerow ($foo, $alignments, $styles);    /* partial sum */
}

/* Make a single row of a list entry consisting of a link.  */
function makelinkle ($row)
{
  preg_match ("~([^ ]+) (.+)~", trim ($row), $parts);
  $URL = $parts [1];
  $desc = $parts [2];
  print <<< EOS
      <li>
        <a href="$URL">$desc</a>
      </li>

EOS;
}

/* Convert a \n delimited string into list entries consisting of a link */
function makelinkles ($glup)
{

  $rows = explode ("\n", $glup);
  $i = 0;
  foreach ($rows as $row)
    makelinkle ($row);
}

/*
 * Make an ingredients table for recipes.
 * $glup is of the form "quantity  +ingredient[  +step]"
 * If step is omitted, it will be replaced by the variable $step.
 * $step is incremented after an empty line.
 */
function makeingredienttable ($glup)
{
  $step = 1;                                    /* step for 3rd parm */
  print <<< EOS
      <table summary="Ingredients">
        <tr>
          <td align="right"><b>quantity</b></td>
          <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
          <td><b>ingredient</b> </td>
          <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
          <td><b>step</b> </td>
        </tr>

EOS;
  $glup = str_replace ("\t", "  ", $glup);
  $rows = explode ("\n", $glup);
  $i = 0;
  $style = array ();                            /* XXX this shouldn't be necessary */
  foreach ($rows as $row)
  {
    if ($row == "")                             /* empty row, */
    {
      $step++;                                  /* new step */
      print <<< EOS
       <tr>
         <td>
         </td>
       </tr>

EOS;
    }
    else
    {
      $row = preg_replace ("|   *|", "\t", $row); /* convert 2 or more spaces to tabs */
      $cols = explode ("\t", $row);
      if (count ($cols) == 3)                   /* step specified */
      {
        $step = $cols [2];                      /* note step number */
        makerow ($row, "rlr", $style);          /* and output the  */
      }
      else
        makerow ($row . "  $step", "rlr", $style); /* use our step number  */
    }
  }
print <<< EOS
      </table>

EOS;
}

/*
 * Make a table from the text supplied in $glup.
 * Optional second parameter specifies the orientation of the columns ("rll",
 * for example).  Default is all columns left-aligned.
 */
function maketable ($glup/* , $alignments */)
{
  if (func_num_args () > 1)
    $alignments = func_get_arg (1);
  else
    $alignments = "l";

  print <<< EOS
<table summary="Automatically generated table">
EOS;
  makerows ($glup, $alignments);
  print <<< EOS
    </table>

EOS;
}

$random_redirects = array ("http://www.google.com",
                           "http://www.yahoo.com",
                           "http://www.FreeBSD.org",
                           "http://en.wikipedia.org/wiki/Saddlebred",
                           "http://www.yahoo.com",
                           "http://www.audioholics.com/news/industry-news/blue-jeans-strikes-back",
                           "http://www.gundaroohorsepark.com.au/",
                           "http://www.microsoft.com",
                           "http://www.disney.com",
                           "http://www.apple.com");

function randomredirect ()
{
  $IP = "192.164.29.203";
/*  $IP = "192.109.197.135";

  echo "<pre>";
  print_r ($_SERVER);
  echo "</pre>"; */
  global $random_redirects;

    if ($_SERVER ['REMOTE_ADDR'] == $IP)
    {
      $site = $random_redirects [array_rand ($random_redirects)];
      error_log ("Redirecting from "
                 . $_SERVER['REQUEST_URI']
                 . " to $site for "
                 . $_SERVER['REMOTE_HOST']);
      print <<< EOS
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="REFRESH" content="0;url=$site" />
  </head>

  <body>
  </body>
</html>
EOS;
      exit ();
    }
}

/*
 * Quote HTML special characters.
 *
 * The second parameter, if present, states whether the text is an href, in
 * which case we need to quote " as well.  By default the answer is "no".
 */
function htmlify ($text/*, $ishref */)
{
  if (func_num_args () > 1)
    $ishref = func_get_arg (1);
  else
    $ishref = 0;
  /* There must be a better way than this */
  $text = preg_replace ('|&|', '&amp;', $text);
  $text = preg_replace ('|<|', '&lt;', $text);
  $text = preg_replace ('|>|', '&gt;', $text);
  $text = preg_replace ('|-|', '&#45;', $text);
  if ($ishref)
  {
    $text = preg_replace ('| |', '%20', $text);
    $text = preg_replace ('|"|', '%22', $text);
    $text = preg_replace ('|\+|', '%2b', $text);
  }
  return $text;
}

/* Make spaces less flexible */
function deflexspace ($text)
{
  /*
   * Replace successive spaces with &nbsp, but leave one so that we can break
   * the line there.  There's got to be an easier way.
   */
  $foo = htmlify ($text);
  $text = preg_replace ('/  /', ' &nbsp;', $text);
/*  $text = preg_replace ('/&nbsp; &nbsp;/', '&nbsp;&nbsp;&nbsp;', $text); */
  return $text;
}

/*
 * Print out the specified lines in <tt> and with no fill or additional wrap.
 */
function nofilltext ($text)
{
  /*
   * Work around browser breakage.  Many browsers interpret -- as the end of a
   * comment, without the trailing >.
   */

  $text = preg_replace ('|-|', '&#45;', $text);
  $lines = explode ("\n", deflexspace ($text));
  $printed = 0;

  print <<< EOT
<div style="text-align:left">
  <tt>

EOT;
  foreach ($lines as $line)
  {
    /*
     * Firefox, at least, does the wrong thing if a line starts with a space and
     * an &nbsp; Try to work around this by replacing the first blank in the
     * string with an &nbsp;, but only if it's the first character in the
     * string; otherwise we might find outselves in the middle of a tag.
     */
    $line = preg_replace ('/^ /', '&nbsp;', $line, 1);
    if ($printed)
      print "<br />";
    $printed = 1;
    print "$line\n";
  }
  print <<< EOT
  </tt>
</div>

EOT;
}

/*
 * Print out the specified text in <tt>, removing HTML special characters.
 */
function inline_literaltext ($text)
{
  print "<tt>" . htmlify ($text) . "</tt>";
}

/*
 * Print out the specified lines in <tt> and with <br/> between lines,
 * removing HTML special characters.
 */
function literaltext ($text)
{
  nofilltext (htmlify ($text));
}

/*
 * Convert the URL to fully qualified form for the sake of RSS output.
 */
function complete_url ($url)
{
  global $servername;

  if (substr ($url, 0, 1) != "/")               /* relative URL, */
  {
    if (! (preg_match ("|^https?:|", $url)))
      $url = $servername . dirname ($_SERVER ["SCRIPT_NAME"]) . "/$url";
  }
  else
    $url = "$servername$url";
  return $url;
}

/* Now we can calculate the URL of this script */
$myurl = complete_url ($_SERVER ["SCRIPT_NAME"]);

/*
 * Generate an <a href="$url">$text</a>, but quote all the special characters in
 * the URL and the text.  If the text is missing, use the URL.
 */
function href ($url/*, $text */)
{
  $text = "";
  $url = complete_url (htmlify ($url, 1));      /* tidy up the URL */
  if (func_num_args () > 1)
    $text = func_get_arg (1);
  if ($text == "")                              /* this could be passed */
    $text = $url;
  print "<a href=\"$url\">$text</a>";
}

function formatmail ($text)
{
  $inpara = 0;                                  /* set when we're inside a <p> tag */
  $quotelevel = 0;                              /* current level of blockquotes */
  $thislevel = 0;                               /* quote level of this line */
  $lines = explode ("\n", $text);
  foreach ($lines as $line)
  {
    if (substr ($line, 0, 1) == '*')            /* Line starts with *: add special processing here */
    {
      if (substr ($line, 1, 1) == ">")          /* photo */
      {
        /* Two different kinds of name */
        $photo = substr ($line, 2);
        $photo = preg_replace ("|.*lemis.com/([^/]+)/.*dirdate=([0-9]*).*&image=([^&#]*).*|",
                               "/\\1/Photos/\\2/\\3", $photo);
        if ($photo == substr ($line, 2))
        $photo = preg_replace ("|.*/([a-z]+)/Photos/([0-9]+)/[a-z]+|",
                               "/\\1/Photos/\\2", $photo);
        print "      ";                         /* this normally comes from the source file */
        showphoto ($photo);
        continue;
      }
      else if (substr ($line, 1, 1) == "-")     /* link */
      {
        $line = preg_replace ("|&|", "&amp;", preg_replace ("|([^ ]*) (.*)|",
                                                          "<a href=\"\\1\">\\2</a>",
                                                          substr ($line, 2)));
        print "$line\n";
        continue;
      }
      else if (substr ($line, 1, 1) == "d")     /* date */
      {
        daytitle (substr ($line, 2));
        continue;
      }
    }
    /* Come here anyway unless we've continued above */
    $line = preg_replace ("~((ftp|https*)://[^ >,]+[^]) >,.])~",
                          "<a href=\"\\1\">\\1</a>",
                          $line );
    $line = ltrim ($line);
    $thislevel = 0;                             /* find quote level for this line */
    while (substr ($line, 0, 1) == '>')
    {
      $line = ltrim (substr ($line, 1));
      $thislevel++;
    }
    while ($thislevel > $quotelevel)            /* more quotes */
    {
      print "      <blockquote>\n";
      $quotelevel++;
    }
    while ($thislevel < $quotelevel)            /* fewer quotes */
    {
      print "      </blockquote>\n";
      $quotelevel--;
    }
    if (strlen ($line))
    {
      if (! $inpara)                            /* start a new paragraph now */
      {
        print "    <p>\n";
        $inpara = 1;
      }
      $line = preg_replace ("|&|", "&amp;",  $line);
      print "$line\n";
    }
    else
    {
      if ($inpara)
      {
        print "    </p>\n";
        $inpara = 0;
      }
    }
  }
}

/* print_r that produces legible output */
function print_pr ($foo)
{
  print "<pre>\n";
  print_r ($foo);
  print "</pre>\n";
}

/*
 * Decide whether to print a section or not, based on selection parameters.
 *
 * This doesn't work with PHP input, which rather limits its usefulness
 */
function displayif ($selection, $text)
{
  /* Cop out for now */
  print $text;
}

/*
 * Decide whether we're local (wwww.lemis.com).
 */
function islocal ()
{
  global $islocal;

  return $islocal;
}

/*
 * Print the text only if we're on wwww.lemis.com.
 * If not ther, and $alt is supplied, print that instead.
 */
function iflocal ($text /*, $alt */)
{
  if (islocal ())
    print $text;
  else if (func_num_args () > 1)
    print func_get_arg (1);
}

/*
 * Check date.  Intended for use in diary entries for delayed publication:
 *
 * <?php if (isafter ("20120505")) : ?>
 * fooo
 * <?php endif; ?>
 */
function isafter ($date)
{
  return (time () > strtotime ($date)) | islocal ();
}

/*
 * Redirect to new local URL.
 * $URL is only the local part of the URL.
 */
function redirect ($URL)
{
  $URL = "http://" . $_SERVER ['HTTP_HOST'] . $URL;
  header ("HTTP/1.0 301 Moved Permanently");
  header ("Location: $URL");
  header ("Connection: close");
  exit ();
}

/*
 * Something like LISP assoc function, using strings.
 * $list contains a string, space and replacement string.
 */
function assoc ($string, $list)
{
  $lines = explode ("\n", $list);
  foreach ($lines as $line)
    if (preg_match ("|^$string (.*)|", $line, $redirect))
      return $redirect [1];
  return FALSE;
}

/* Kludge to decide whether a crawler is talking to us */
function iscrawler ()
{
  $crawlers = "/crawl.yahoo.net$|googlebot.com$|search.msn.com$/";
  /* martins says: */
  $crawlers = "/slurp|bot|spider|crawler/i";
/* Debug  $crawlers = "/crawl.yahoo.net$|googlebot.com$|search.msn.com$|dereel.lemis.com$/"; */
  if (array_key_exists ("HTTP_USER_AGENT", $_SERVER))
  {
    $result =  preg_match ($crawlers, $_SERVER ["HTTP_USER_AGENT"]);
    $REQUESTER = $_SERVER ["HTTP_USER_AGENT"];
    if ($result)
    {
      if (0)
      {
        $REDIRECT = $_SERVER ["SCRIPT_NAME"];
        $URL = $_SERVER ["REQUEST_URI"];
        $SERVER = $_SERVER["HTTP_HOST"];
        $FULLURL = "http://" . $_SERVER ["HTTP_HOST"] . $URL;
        mail ("grog@lemis.com",
              "Crawler request from $REQUESTER",
              <<< EOS
Referenced URL: $FULLURL
Redirected to:  $REDIRECT
EOS
              );
      }
    }
    return $result;
  }
  else
    return 0;
}

function print_info ($PARM)
{
  print ("<pre>\n");
  print_r ($_SERVER);
  print "PARM: $PARM";
  print ("</pre>\n");
}

/*
 * Generalized redirect.  This matches the current file name, so we can use a
 * single multi-linked file to represent lots of dead pages.
 */
function redirectme ($renames)
{
  if (0)                                        /* debug */
    $NODELAY=100;
  else
    $NODELAY=5;
  $URL = $_SERVER ["REQUEST_URI"];
  if ($PURL = assoc ($URL, $renames))
  {
    print <<< EOS
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
	  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta http-equiv="refresh" content="$NODELAY;URL=$PURL" />
  </head>

    <body>
      <p>
	The URL you have specified, $URL,
	no longer exists.  It has been replaced by $PURL.
	Please update your links.
      </p>

      <p>
	You will be redirected to $PURL in $NODELAY seconds.
      </p>
    </body>
  </html>
EOS;
  }
  else
    print <<< EOS
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
	  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
  </head>

    <body>
      <p>
	The URL you have specified, $URL,
      no longer exists, and we don&apos;t have a replacement.
	This is an error on our site.  Sorry.
      </p>
    </body>
  </html>
EOS;
  exit ();
}

function google_translate ($url, $from /*, $to */)
{
  if (func_num_args () > 2)
    $to = func_get_arg (2);
  else
    $to = "en";
  $url = htmlify ($url);
  print "<a href=\"http://translate.google.com/translate?sl=$from&tl=$to&ie=UTF-8&u=$url\">Google Translate</a>";
}
